@@ -5,35 +5,34 @@
 """
 Base code for the Qt backends. Note that this is *not* (anymore) a
 backend by itself! One has to explicitly use either PySide, PyQt4 or
-PySide2, PyQt5. Note that the automatic backend selection prefers
+PySide2, PyQt5 or PyQt6. Note that the automatic backend selection prefers
 a GUI toolkit that is already imported.
 
-The _pyside, _pyqt4, _pyside2 and _pyqt5 modules will import * from
-this module, and also keep a ref to the module object. Note that if
-two of the backends are used, this module is actually reloaded. This
-is a sorts of poor mans "subclassing" to get a working version for
-both backends using the same code.
+The _pyside, _pyqt4, _pyside2, _pyqt5 and _pyside6 modules will
+import * from this module, and also keep a ref to the module object.
+Note that if two of the backends are used, this module is actually
+reloaded. This is a sorts of poor mans "subclassing" to get a working
+version for both backends using the same code.
 
 Note that it is strongly discouraged to use the
-PySide/PyQt4/PySide2/PyQt5 backends simultaneously. It is known to
-cause unpredictable behavior and segfaults.
+PySide/PyQt4/PySide2/PyQt5/PySide6 backends simultaneously. It is
+known to cause unpredictable behavior and segfaults.
 """
 
 from __future__ import division
 
 from time import sleep, time
+import math
 import os
 import sys
 import atexit
 import ctypes
-from distutils.version import LooseVersion
+from packaging.version import Version
 
 from ...util import logger
 from ..base import (BaseApplicationBackend, BaseCanvasBackend,
                     BaseTimerBackend)
 from ...util import keys
-from ...ext.six import text_type
-from ...ext.six import string_types
 from ... import config
 from . import qt_lib
 
@@ -56,7 +55,7 @@ elif sys.platform.startswith('win'):
 
 def _check_imports(lib):
     # Make sure no conflicting libraries have been imported.
-    libs = ['PyQt4', 'PyQt5', 'PySide', 'PySide2']
+    libs = ['PyQt4', 'PyQt5', 'PyQt6', 'PySide', 'PySide2', 'PySide6']
     libs.remove(lib)
     for lib2 in libs:
         lib2 += '.QtCore'
@@ -64,10 +63,24 @@ def _check_imports(lib):
             raise RuntimeError("Refusing to import %s because %s is already "
                                "imported." % (lib, lib2))
 
+
+def _get_event_xy(ev):
+    # QT6 (and the Python bindings like PyQt6, PySide6) report position differently from previous versions
+    if hasattr(ev, 'pos'):
+        posx, posy = ev.pos().x(), ev.pos().y()
+    else:
+        # Compatibility for PySide6 / PyQt6
+        posx, posy = ev.position().x(), ev.position().y()
+
+    return posx, posy
+
+
 # Get what qt lib to try. This tells us wheter this module is imported
 # via _pyside or _pyqt4 or _pyqt5
 QGLWidget = object
 QT5_NEW_API = False
+PYSIDE6_API = False
+PYQT6_API = False
 if qt_lib == 'pyqt4':
     _check_imports('PyQt4')
     if not USE_EGL:
@@ -78,7 +91,7 @@ elif qt_lib == 'pyqt5':
     _check_imports('PyQt5')
     if not USE_EGL:
         from PyQt5.QtCore import QT_VERSION_STR
-        if LooseVersion(QT_VERSION_STR) >= '5.4.0':
+        if Version(QT_VERSION_STR) >= Version('5.4.0'):
             from PyQt5.QtWidgets import QOpenGLWidget as QGLWidget
             from PyQt5.QtGui import QSurfaceFormat as QGLFormat
             QT5_NEW_API = True
@@ -86,11 +99,35 @@ elif qt_lib == 'pyqt5':
             from PyQt5.QtOpenGL import QGLWidget, QGLFormat
     from PyQt5 import QtGui, QtCore, QtWidgets, QtTest
     QWidget, QApplication = QtWidgets.QWidget, QtWidgets.QApplication  # Compat
+elif qt_lib == 'pyqt6':
+    _check_imports('PyQt6')
+    if not USE_EGL:
+        from PyQt6.QtCore import QT_VERSION_STR
+        if Version(QT_VERSION_STR) >= Version('6.0.0'):
+            from PyQt6.QtOpenGLWidgets import QOpenGLWidget as QGLWidget
+            from PyQt6.QtGui import QSurfaceFormat as QGLFormat
+            PYQT6_API = True
+        else:
+            from PyQt6.QtOpenGL import QGLWidget, QGLFormat
+    from PyQt6 import QtGui, QtCore, QtWidgets, QtTest
+    QWidget, QApplication = QtWidgets.QWidget, QtWidgets.QApplication  # Compat
+elif qt_lib == 'pyside6':
+    _check_imports('PySide6')
+    if not USE_EGL:
+        from PySide6.QtCore import __version__ as QT_VERSION_STR
+        if Version(QT_VERSION_STR) >= Version('6.0.0'):
+            from PySide6.QtOpenGLWidgets import QOpenGLWidget as QGLWidget
+            from PySide6.QtGui import QSurfaceFormat as QGLFormat
+            PYSIDE6_API = True
+        else:
+            from PySide6.QtOpenGL import QGLWidget, QGLFormat
+    from PySide6 import QtGui, QtCore, QtWidgets, QtTest
+    QWidget, QApplication = QtWidgets.QWidget, QtWidgets.QApplication  # Compat
 elif qt_lib == 'pyside2':
     _check_imports('PySide2')
     if not USE_EGL:
         from PySide2.QtCore import __version__ as QT_VERSION_STR
-        if LooseVersion(QT_VERSION_STR) >= '5.4.0':
+        if Version(QT_VERSION_STR) >= Version('5.4.0'):
             from PySide2.QtWidgets import QOpenGLWidget as QGLWidget
             from PySide2.QtGui import QSurfaceFormat as QGLFormat
             QT5_NEW_API = True
@@ -114,47 +151,58 @@ else:
 #   Shift  50, 62
 #   Ctrl   37, 105
 #   Alt    64, 108
+qt_keys = QtCore.Qt.Key if qt_lib == 'pyqt6' else QtCore.Qt
 KEYMAP = {
-    QtCore.Qt.Key_Shift: keys.SHIFT,
-    QtCore.Qt.Key_Control: keys.CONTROL,
-    QtCore.Qt.Key_Alt: keys.ALT,
-    QtCore.Qt.Key_AltGr: keys.ALT,
-    QtCore.Qt.Key_Meta: keys.META,
+    qt_keys.Key_Shift: keys.SHIFT,
+    qt_keys.Key_Control: keys.CONTROL,
+    qt_keys.Key_Alt: keys.ALT,
+    qt_keys.Key_AltGr: keys.ALT,
+    qt_keys.Key_Meta: keys.META,
 
-    QtCore.Qt.Key_Left: keys.LEFT,
-    QtCore.Qt.Key_Up: keys.UP,
-    QtCore.Qt.Key_Right: keys.RIGHT,
-    QtCore.Qt.Key_Down: keys.DOWN,
-    QtCore.Qt.Key_PageUp: keys.PAGEUP,
-    QtCore.Qt.Key_PageDown: keys.PAGEDOWN,
+    qt_keys.Key_Left: keys.LEFT,
+    qt_keys.Key_Up: keys.UP,
+    qt_keys.Key_Right: keys.RIGHT,
+    qt_keys.Key_Down: keys.DOWN,
+    qt_keys.Key_PageUp: keys.PAGEUP,
+    qt_keys.Key_PageDown: keys.PAGEDOWN,
 
-    QtCore.Qt.Key_Insert: keys.INSERT,
-    QtCore.Qt.Key_Delete: keys.DELETE,
-    QtCore.Qt.Key_Home: keys.HOME,
-    QtCore.Qt.Key_End: keys.END,
+    qt_keys.Key_Insert: keys.INSERT,
+    qt_keys.Key_Delete: keys.DELETE,
+    qt_keys.Key_Home: keys.HOME,
+    qt_keys.Key_End: keys.END,
 
-    QtCore.Qt.Key_Escape: keys.ESCAPE,
-    QtCore.Qt.Key_Backspace: keys.BACKSPACE,
+    qt_keys.Key_Escape: keys.ESCAPE,
+    qt_keys.Key_Backspace: keys.BACKSPACE,
 
-    QtCore.Qt.Key_F1: keys.F1,
-    QtCore.Qt.Key_F2: keys.F2,
-    QtCore.Qt.Key_F3: keys.F3,
-    QtCore.Qt.Key_F4: keys.F4,
-    QtCore.Qt.Key_F5: keys.F5,
-    QtCore.Qt.Key_F6: keys.F6,
-    QtCore.Qt.Key_F7: keys.F7,
-    QtCore.Qt.Key_F8: keys.F8,
-    QtCore.Qt.Key_F9: keys.F9,
-    QtCore.Qt.Key_F10: keys.F10,
-    QtCore.Qt.Key_F11: keys.F11,
-    QtCore.Qt.Key_F12: keys.F12,
+    qt_keys.Key_F1: keys.F1,
+    qt_keys.Key_F2: keys.F2,
+    qt_keys.Key_F3: keys.F3,
+    qt_keys.Key_F4: keys.F4,
+    qt_keys.Key_F5: keys.F5,
+    qt_keys.Key_F6: keys.F6,
+    qt_keys.Key_F7: keys.F7,
+    qt_keys.Key_F8: keys.F8,
+    qt_keys.Key_F9: keys.F9,
+    qt_keys.Key_F10: keys.F10,
+    qt_keys.Key_F11: keys.F11,
+    qt_keys.Key_F12: keys.F12,
 
-    QtCore.Qt.Key_Space: keys.SPACE,
-    QtCore.Qt.Key_Enter: keys.ENTER,
-    QtCore.Qt.Key_Return: keys.ENTER,
-    QtCore.Qt.Key_Tab: keys.TAB,
+    qt_keys.Key_Space: keys.SPACE,
+    qt_keys.Key_Enter: keys.ENTER,
+    qt_keys.Key_Return: keys.ENTER,
+    qt_keys.Key_Tab: keys.TAB,
 }
-BUTTONMAP = {0: 0, 1: 1, 2: 2, 4: 3, 8: 4, 16: 5}
+if PYQT6_API or PYSIDE6_API:
+    BUTTONMAP = {
+        QtCore.Qt.MouseButton.NoButton: 0,
+        QtCore.Qt.MouseButton.LeftButton: 1,
+        QtCore.Qt.MouseButton.RightButton: 2,
+        QtCore.Qt.MouseButton.MiddleButton: 3,
+        QtCore.Qt.MouseButton.BackButton: 4,
+        QtCore.Qt.MouseButton.ForwardButton: 5
+    }
+else:
+    BUTTONMAP = {0: 0, 1: 1, 2: 2, 4: 3, 8: 4, 16: 5}
 
 
 # Properly log Qt messages
@@ -162,7 +210,7 @@ def message_handler(*args):
 
     if qt_lib in ("pyqt4", "pyside"):
         msg_type, msg = args
-    elif qt_lib in ("pyqt5", "pyside2"):  # Is this correct for pyside2?
+    elif qt_lib in ("pyqt5", "pyqt6", "pyside2", "pyside6"):  # Is this correct for pyside2?
         msg_type, context, msg = args
     elif qt_lib:
         raise RuntimeError("Invalid value for qt_lib %r." % qt_lib)
@@ -183,7 +231,7 @@ def message_handler(*args):
         if msg.startswith(item):
             return
 
-    msg = msg.decode() if not isinstance(msg, string_types) else msg
+    msg = msg.decode() if not isinstance(msg, str) else msg
     logger.warning(msg)
 
 
@@ -195,13 +243,13 @@ def use_shared_contexts():
 
     """
     forced_env_var = os.getenv('VISPY_PYQT5_SHARE_CONTEXT', 'false').lower() == 'true'
-    return not QT5_NEW_API or forced_env_var
+    return not (QT5_NEW_API or PYSIDE6_API or PYQT6_API) or forced_env_var
 
 
 try:
     QtCore.qInstallMsgHandler(message_handler)
 except AttributeError:
-    QtCore.qInstallMessageHandler(message_handler)  # PyQt5
+    QtCore.qInstallMessageHandler(message_handler)  # PyQt5, PyQt6
 
 
 # -------------------------------------------------------------- capability ---
@@ -235,6 +283,9 @@ def _set_config(c):
         # Qt5 >= 5.4.0 - below options automatically enabled if nonzero.
         glformat.setSwapBehavior(glformat.DoubleBuffer if c['double_buffer']
                                  else glformat.SingleBuffer)
+    elif PYQT6_API or PYSIDE6_API:
+        glformat.setSwapBehavior(glformat.SwapBehavior.DoubleBuffer if c['double_buffer']
+                                 else glformat.SwapBehavior.SingleBuffer)
     else:
         # Qt4 and Qt5 < 5.4.0 - buffers must be explicitly requested.
         glformat.setAccum(False)
@@ -258,9 +309,11 @@ class ApplicationBackend(BaseApplicationBackend):
     def __init__(self):
         BaseApplicationBackend.__init__(self)
         # sharing is currently buggy and causes segmentation faults for tests with PyQt 5.6
-        if QT5_NEW_API and use_shared_contexts():
+        if (QT5_NEW_API or PYSIDE6_API) and use_shared_contexts():
             # For Qt5 >= 5.4.0 - Enable sharing of context between windows.
             QApplication.setAttribute(QtCore.Qt.AA_ShareOpenGLContexts, True)
+        elif PYQT6_API and use_shared_contexts():
+            QApplication.setAttribute(QtCore.Qt.ApplicationAttribute.AA_ShareOpenGLContexts, True)
 
     def _vispy_get_backend_name(self):
         name = QtCore.__name__.split('.')[0]
@@ -268,7 +321,9 @@ class ApplicationBackend(BaseApplicationBackend):
 
     def _vispy_process_events(self):
         app = self._vispy_get_native_app()
-        app.flush()
+        # sendPostedEvents replaces flush which has been removed from Qt6.0+
+        # This should be compatible with Qt4.x and Qt5.x
+        app.sendPostedEvents()
         app.processEvents()
 
     def _vispy_run(self):
@@ -276,7 +331,9 @@ class ApplicationBackend(BaseApplicationBackend):
         if hasattr(app, '_in_event_loop') and app._in_event_loop:
             pass  # Already in event loop
         else:
-            return app.exec_()
+            # app.exec_() for PyQt <=5 and app.exec() for PyQt >=5
+            exec_func = app.exec if hasattr(app, "exec") else app.exec_
+            return exec_func()
 
     def _vispy_quit(self):
         return self._vispy_get_native_app().quit()
@@ -306,9 +363,8 @@ def _get_qpoint_pos(pos):
 class QtBaseCanvasBackend(BaseCanvasBackend):
     """Base functionality of Qt backend. No OpenGL Stuff."""
 
-    # args are for BaseCanvasBackend, kwargs are for us.
-    def __init__(self, *args, **kwargs):
-        BaseCanvasBackend.__init__(self, *args)
+    def __init__(self, vispy_canvas, **kwargs):
+        BaseCanvasBackend.__init__(self, vispy_canvas)
         # Maybe to ensure that exactly all arguments are passed?
         p = self._process_backend_kwargs(kwargs)
         self._initialized = False
@@ -355,13 +411,10 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             # either not PyQt5 backend or no parent window available
             pass
 
-        # Activate touch and gesture.
-        # NOTE: we only activate touch on OS X because there seems to be
-        # problems on Ubuntu computers with touchscreen.
-        # See https://github.com/vispy/vispy/pull/1143
-        if sys.platform == 'darwin':
-            self.setAttribute(QtCore.Qt.WA_AcceptTouchEvents)
-            self.grabGesture(QtCore.Qt.PinchGesture)
+        # QNativeGestureEvent does not keep track of last or total
+        # values like QGestureEvent does
+        self._native_gesture_scale_values = []
+        self._native_gesture_rotation_values = []
 
     def screen_changed(self, new_screen):
         """Window moved from one display to another, resize canvas.
@@ -442,7 +495,7 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             return
         self._vispy_mouse_press(
             native=ev,
-            pos=(ev.pos().x(), ev.pos().y()),
+            pos=_get_event_xy(ev),
             button=BUTTONMAP.get(ev.button(), 0),
             modifiers=self._modifiers(ev),
         )
@@ -452,7 +505,7 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             return
         self._vispy_mouse_release(
             native=ev,
-            pos=(ev.pos().x(), ev.pos().y()),
+            pos=_get_event_xy(ev),
             button=BUTTONMAP[ev.button()],
             modifiers=self._modifiers(ev),
         )
@@ -462,7 +515,7 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             return
         self._vispy_mouse_double_click(
             native=ev,
-            pos=(ev.pos().x(), ev.pos().y()),
+            pos=_get_event_xy(ev),
             button=BUTTONMAP.get(ev.button(), 0),
             modifiers=self._modifiers(ev),
         )
@@ -472,7 +525,7 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             return
         self._vispy_mouse_move(
             native=ev,
-            pos=(ev.pos().x(), ev.pos().y()),
+            pos=_get_event_xy(ev),
             modifiers=self._modifiers(ev),
         )
 
@@ -487,14 +540,14 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             else:
                 deltay = ev.delta() / 120.0
         else:
-            # PyQt5
+            # PyQt5 / PyQt6
             delta = ev.angleDelta()
             deltax, deltay = delta.x() / 120.0, delta.y() / 120.0
         # Emit event
         self._vispy_canvas.events.mouse_wheel(
             native=ev,
             delta=(deltax, deltay),
-            pos=(ev.pos().x(), ev.pos().y()),
+            pos=_get_event_xy(ev),
             modifiers=self._modifiers(ev),
         )
 
@@ -504,38 +557,81 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
     def keyReleaseEvent(self, ev):
         self._keyEvent(self._vispy_canvas.events.key_release, ev)
 
+    def _handle_native_gesture_event(self, ev):
+        if self._vispy_canvas is None:
+            return
+        t = ev.gestureType()
+        # this is a workaround for what looks like a Qt bug where
+        # QNativeGestureEvent gives the wrong local position.
+        # See: https://bugreports.qt.io/browse/QTBUG-59595
+        try:
+            pos = self.mapFromGlobal(ev.globalPosition().toPoint())
+        except AttributeError:
+            # globalPos is deprecated in Qt6
+            pos = self.mapFromGlobal(ev.globalPos())
+        pos = pos.x(), pos.y()
+
+        if t == QtCore.Qt.NativeGestureType.BeginNativeGesture:
+            self._vispy_canvas.events.touch(
+                type='gesture_begin',
+                pos=_get_event_xy(ev),
+            )
+        elif t == QtCore.Qt.NativeGestureType.EndNativeGesture:
+            self._native_touch_total_rotation = []
+            self._native_touch_total_scale = []
+            self._vispy_canvas.events.touch(
+                type='gesture_end',
+                pos=_get_event_xy(ev),
+            )
+        elif t == QtCore.Qt.NativeGestureType.RotateNativeGesture:
+            angle = ev.value()
+            last_angle = (
+                self._native_gesture_rotation_values[-1]
+                if self._native_gesture_rotation_values
+                else None
+            )
+            self._native_gesture_rotation_values.append(angle)
+            total_rotation_angle = math.fsum(self._native_gesture_rotation_values)
+            self._vispy_canvas.events.touch(
+                type="gesture_rotate",
+                pos=pos,
+                rotation=angle,
+                last_rotation=last_angle,
+                total_rotation_angle=total_rotation_angle,
+            )
+        elif t == QtCore.Qt.NativeGestureType.ZoomNativeGesture:
+            scale = ev.value()
+            last_scale = (
+                self._native_gesture_scale_values[-1]
+                if self._native_gesture_scale_values
+                else None
+            )
+            self._native_gesture_scale_values.append(scale)
+            total_scale_factor = math.fsum(self._native_gesture_scale_values)
+            self._vispy_canvas.events.touch(
+                type="gesture_zoom",
+                pos=pos,
+                last_scale=last_scale,
+                scale=scale,
+                total_scale_factor=total_scale_factor,
+            )
+        # QtCore.Qt.NativeGestureType.PanNativeGesture
+        # Qt6 docs seem to imply this is only supported on Wayland but I have
+        # not been able to test it.
+        # Two finger pan events are anyway converted to scroll/wheel events.
+        # On macOS, more fingers are usually swallowed by the OS (by spaces,
+        # mission control, etc.).
+
     def event(self, ev):
         out = super(QtBaseCanvasBackend, self).event(ev)
-        t = ev.type()
-        # Two-finger pinch.
-        if (t == QtCore.QEvent.TouchBegin):
-            self._vispy_canvas.events.touch(type='begin')
-        if (t == QtCore.QEvent.TouchEnd):
-            self._vispy_canvas.events.touch(type='end')
-        if (t == QtCore.QEvent.Gesture):
-            gesture = ev.gesture(QtCore.Qt.PinchGesture)
-            if gesture:
-                (x, y) = _get_qpoint_pos(gesture.centerPoint())
-                scale = gesture.scaleFactor()
-                last_scale = gesture.lastScaleFactor()
-                rotation = gesture.rotationAngle()
-                self._vispy_canvas.events.touch(type='pinch',
-                                                pos=(x, y),
-                                                last_pos=None,
-                                                scale=scale,
-                                                last_scale=last_scale,
-                                                rotation=rotation,
-                                                )
-        # General touch event.
-        elif (t == QtCore.QEvent.TouchUpdate):
-            points = ev.touchPoints()
-            # These variables are lists of (x, y) coordinates.
-            pos = [_get_qpoint_pos(p.pos()) for p in points]
-            lpos = [_get_qpoint_pos(p.lastPos()) for p in points]
-            self._vispy_canvas.events.touch(type='touch',
-                                            pos=pos,
-                                            last_pos=lpos,
-                                            )
+
+        # QNativeGestureEvent is Qt 5+
+        if (
+            (QT5_NEW_API or PYSIDE6_API or PYQT6_API)
+            and isinstance(ev, QtGui.QNativeGestureEvent)
+        ):
+            self._handle_native_gesture_event(ev)
+
         return out
 
     def _keyEvent(self, func, ev):
@@ -543,22 +639,23 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
         key = int(ev.key())
         if key in KEYMAP:
             key = KEYMAP[key]
-        elif key >= 32 and key <= 127:
+        elif 32 <= key <= 127:
             key = keys.Key(chr(key))
         else:
             key = None
         mod = self._modifiers(ev)
-        func(native=ev, key=key, text=text_type(ev.text()), modifiers=mod)
+        func(native=ev, key=key, text=str(ev.text()), modifiers=mod)
 
     def _modifiers(self, event):
         # Convert the QT modifier state into a tuple of active modifier keys.
         mod = ()
         qtmod = event.modifiers()
-        for q, v in ([QtCore.Qt.ShiftModifier, keys.SHIFT],
-                     [QtCore.Qt.ControlModifier, keys.CONTROL],
-                     [QtCore.Qt.AltModifier, keys.ALT],
-                     [QtCore.Qt.MetaModifier, keys.META]):
-            if q & qtmod:
+        qt_keyboard_modifiers = QtCore.Qt.KeyboardModifier if PYQT6_API else QtCore.Qt
+        for q, v in ([qt_keyboard_modifiers.ShiftModifier, keys.SHIFT],
+                     [qt_keyboard_modifiers.ControlModifier, keys.CONTROL],
+                     [qt_keyboard_modifiers.AltModifier, keys.ALT],
+                     [qt_keyboard_modifiers.MetaModifier, keys.META]):
+            if qtmod & q:
                 mod += (v,)
         return mod
 
@@ -600,20 +697,23 @@ class CanvasBackendEgl(QtBaseCanvasBackend, QWidget):
             self._native_context = p.context.shared.ref._native_context
 
         # Init widget
+        qt_window_types = QtCore.Qt.WindowType if PYQT6_API else QtCore.Qt
         if p.always_on_top or not p.decorate:
             hint = 0
-            hint |= 0 if p.decorate else QtCore.Qt.FramelessWindowHint
-            hint |= QtCore.Qt.WindowStaysOnTopHint if p.always_on_top else 0
+            hint |= 0 if p.decorate else qt_window_types.FramelessWindowHint
+            hint |= qt_window_types.WindowStaysOnTopHint if p.always_on_top else 0
         else:
-            hint = QtCore.Qt.Widget  # can also be a window type
+            hint = qt_window_types.Widget  # can also be a window type
+
         QWidget.__init__(self, p.parent, hint)
 
+        qt_window_attributes = QtCore.Qt.WidgetAttribute if PYQT6_API else QtCore.Qt
         if 0:  # IS_LINUX or IS_RPI:
             self.setAutoFillBackground(False)
-            self.setAttribute(QtCore.Qt.WA_NoSystemBackground, True)
-            self.setAttribute(QtCore.Qt.WA_OpaquePaintEvent, True)
+            self.setAttribute(qt_window_attributes.WA_NoSystemBackground, True)
+            self.setAttribute(qt_window_attributes.WA_OpaquePaintEvent, True)
         elif IS_WIN:
-            self.setAttribute(QtCore.Qt.WA_PaintOnScreen, True)
+            self.setAttribute(qt_window_attributes.WA_PaintOnScreen, True)
             self.setAutoFillBackground(False)
 
         # Init surface
@@ -623,8 +723,7 @@ class CanvasBackendEgl(QtBaseCanvasBackend, QWidget):
         self._initialized = True
 
     def get_window_id(self):
-        """ Get the window id of a PySide Widget. Might also work for PyQt4.
-        """
+        """Get the window id of a PySide Widget. Might also work for PyQt4."""
         # Get Qt win id
         winid = self.winId()
 
@@ -637,9 +736,9 @@ class CanvasBackendEgl(QtBaseCanvasBackend, QWidget):
 
         # Get window id from stupid capsule thingy
         # http://translate.google.com/translate?hl=en&sl=zh-CN&u=http://www.cnb
-        #logs.com/Shiren-Y/archive/2011/04/06/2007288.html&prev=/search%3Fq%3Dp
+        # logs.com/Shiren-Y/archive/2011/04/06/2007288.html&prev=/search%3Fq%3Dp
         # yside%2Bdirectx%26client%3Dfirefox-a%26hs%3DIsJ%26rls%3Dorg.mozilla:n
-        #l:official%26channel%3Dfflb%26biw%3D1366%26bih%3D614
+        # l:official%26channel%3Dfflb%26biw%3D1366%26bih%3D614
         # Prepare
         ctypes.pythonapi.PyCapsule_GetName.restype = ctypes.c_char_p
         ctypes.pythonapi.PyCapsule_GetName.argtypes = [ctypes.py_object]
@@ -728,13 +827,15 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
                 raise RuntimeError('Cannot use vispy to share context and '
                                    'use built-in shareWidget.')
 
+        qt_window_types = QtCore.Qt.WindowType if PYQT6_API else QtCore.Qt
         if p.always_on_top or not p.decorate:
             hint = 0
-            hint |= 0 if p.decorate else QtCore.Qt.FramelessWindowHint
-            hint |= QtCore.Qt.WindowStaysOnTopHint if p.always_on_top else 0
+            hint |= 0 if p.decorate else qt_window_types.FramelessWindowHint
+            hint |= qt_window_types.WindowStaysOnTopHint if p.always_on_top else 0
         else:
-            hint = QtCore.Qt.Widget  # can also be a window type
-        if QT5_NEW_API:
+            hint = qt_window_types.Widget  # can also be a window type
+
+        if QT5_NEW_API or PYSIDE6_API or PYQT6_API:
             # Qt5 >= 5.4.0 - sharing is automatic
             QGLWidget.__init__(self, p.parent, hint)
 
@@ -759,19 +860,20 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
 
         self.setFormat(glformat)
         self._initialized = True
-        if not QT5_NEW_API and not self.isValid():
+        if not QT5_NEW_API and not PYSIDE6_API and not PYQT6_API and not self.isValid():
             # On Qt5 >= 5.4.0, isValid is only true once the widget is shown
             raise RuntimeError('context could not be created')
-        if not QT5_NEW_API:
+        if not QT5_NEW_API and not PYSIDE6_API and not PYQT6_API:
             # to make consistent with other backends
             self.setAutoBufferSwap(False)
-        self.setFocusPolicy(QtCore.Qt.WheelFocus)
+        qt_focus_policies = QtCore.Qt.FocusPolicy if PYQT6_API else QtCore.Qt
+        self.setFocusPolicy(qt_focus_policies.WheelFocus)
 
     def _vispy_close(self):
         # Force the window or widget to shut down
         self.close()
         self.doneCurrent()
-        if not QT5_NEW_API:
+        if not QT5_NEW_API and not PYSIDE6_API and not PYQT6_API:
             self.context().reset()
         if self._vispy_canvas is not None:
             self._vispy_canvas.app.process_events()
@@ -787,14 +889,14 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
         # Swap front and back buffer
         if self._vispy_canvas is None:
             return
-        if QT5_NEW_API:
+        if QT5_NEW_API or PYSIDE6_API or PYQT6_API:
             ctx = self.context()
             ctx.swapBuffers(ctx.surface())
         else:
             self.swapBuffers()
 
     def _vispy_get_fb_bind_location(self):
-        if QT5_NEW_API:
+        if QT5_NEW_API or PYSIDE6_API or PYQT6_API:
             return self.defaultFramebufferObject()
         else:
             return QtBaseCanvasBackend._vispy_get_fb_bind_location(self)
@@ -810,9 +912,10 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
         if hasattr(self, 'devicePixelRatio'):
             # We take into account devicePixelRatio, which is non-unity on
             # e.g HiDPI displays.
+            # self.devicePixelRatio() is a float and should have been in Qt5 according to the documentation
             ratio = self.devicePixelRatio()
-            w = w * ratio
-            h = h * ratio
+            w = int(w * ratio)
+            h = int(h * ratio)
         self._vispy_set_physical_size(w, h)
         self._vispy_canvas.events.resize(size=(self.width(), self.height()),
                                          physical_size=(w, h))
@@ -824,6 +927,16 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
         self._vispy_canvas.set_current()
         self._vispy_canvas.events.draw(region=None)
 
+        # Clear the alpha channel with QOpenGLWidget (Qt >= 5.4), otherwise the
+        # window is translucent behind non-opaque objects.
+        # Reference:  MRtrix3/mrtrix3#266
+        if QT5_NEW_API or PYSIDE6_API or PYQT6_API:
+            context = self._vispy_canvas.context
+            context.set_color_mask(False, False, False, True)
+            context.clear(color=True, depth=False, stencil=False)
+            context.set_color_mask(True, True, True, True)
+            context.flush()
+
 
 # Select CanvasBackend
 if USE_EGL:
@@ -846,7 +959,7 @@ class TimerBackend(BaseTimerBackend, QtCore.QTimer):
         self.timeout.connect(self._vispy_timeout)
 
     def _vispy_start(self, interval):
-        self.start(interval * 1000.)
+        self.start(int(interval * 1000))
 
     def _vispy_stop(self):
         self.stop()
