@@ -4,8 +4,6 @@
 
 import numpy as np
 
-from ..ext.six.moves import xrange
-
 
 def _fix_colors(colors):
     colors = np.asarray(colors)
@@ -21,6 +19,56 @@ def _fix_colors(colors):
     return colors
 
 
+def _compute_face_normals(vertices):
+    if vertices.shape[1:] != (3, 3):
+        raise ValueError("Expected (N, 3, 3) array of vertices repeated on"
+                         f" the triangle corners, got {vertices.shape}.")
+    edges1 = vertices[:, 1] - vertices[:, 0]
+    edges2 = vertices[:, 2] - vertices[:, 0]
+    return np.cross(edges1, edges2)
+
+
+def _repeat_face_normals_on_corners(normals):
+    if normals.shape[1:] != (3,):
+        raise ValueError("Expected (F, 3) array of face normals, got"
+                         f" {normals.shape}.")
+    n_corners_in_face = 3
+    new_shape = (normals.shape[0], n_corners_in_face, normals.shape[1])
+    return np.repeat(normals, n_corners_in_face, axis=0).reshape(new_shape)
+
+
+def _compute_vertex_normals(face_normals, faces, vertices):
+    if face_normals.shape[1:] != (3,):
+        raise ValueError("Expected (F, 3) array of face normals, got"
+                         f" {face_normals.shape}.")
+    if faces.shape[1:] != (3,):
+        raise ValueError("Expected (F, 3) array of face vertex indices, got"
+                         f" {faces.shape}.")
+    if vertices.shape[1:] != (3,):
+        raise ValueError("Expected (N, 3) array of vertices, got"
+                         f" {vertices.shape}.")
+
+    vertex_normals = np.zeros_like(vertices)
+    n_corners_in_triangle = 3
+    face_normals_repeated_on_corners = np.repeat(face_normals,
+                                                 n_corners_in_triangle,
+                                                 axis=0)
+    # NOTE: The next line is equivalent to
+    #
+    #   vertex_normals[self._faces.ravel()] += face_normals_repeated_on_corners
+    #
+    # except that it accumulates the values from the right hand side at
+    # repeated indices on the left hand side, instead of overwritting them,
+    # like in the above.
+    np.add.at(vertex_normals, faces.ravel(), face_normals_repeated_on_corners)
+
+    norms = np.sqrt((vertex_normals**2).sum(axis=1))
+    nonzero_norms = norms > 0
+    vertex_normals[nonzero_norms] /= norms[nonzero_norms][:, None]
+
+    return vertex_normals
+
+
 class MeshData(object):
     """
     Class for storing and operating on 3D mesh data.
@@ -134,7 +182,6 @@ class MeshData(object):
         edges : ndarray
             The edges.
         """
-
         if indexed is None:
             if self._edges is None:
                 self._compute_edges(indexed=None)
@@ -144,7 +191,7 @@ class MeshData(object):
                 self._compute_edges(indexed='faces')
             return self._edges_indexed_by_faces
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
     def set_faces(self, faces):
         """Set the faces
@@ -194,7 +241,7 @@ class MeshData(object):
                     self._vertices[self.get_faces()]
             return self._vertices_indexed_by_faces
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
     def get_bounds(self):
         """Get the mesh bounds
@@ -237,7 +284,7 @@ class MeshData(object):
             if verts is not None:
                 self._vertices_indexed_by_faces = verts
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
         if reset_normals:
             self.reset_normals()
@@ -250,7 +297,8 @@ class MeshData(object):
 
     def has_face_indexed_data(self):
         """Return True if this object already has vertex positions indexed
-        by face"""
+        by face
+        """
         return self._vertices_indexed_by_faces is not None
 
     def has_edge_indexed_data(self):
@@ -295,22 +343,19 @@ class MeshData(object):
         normals : ndarray
             The normals.
         """
-        if self._face_normals is None:
-            v = self.get_vertices(indexed='faces')
-            self._face_normals = np.cross(v[:, 1] - v[:, 0],
-                                          v[:, 2] - v[:, 0])
+        if indexed not in (None, 'faces'):
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
-        if indexed is None:
-            return self._face_normals
-        elif indexed == 'faces':
-            if self._face_normals_indexed_by_faces is None:
-                norms = np.empty((self._face_normals.shape[0], 3, 3),
-                                 dtype=np.float32)
-                norms[:] = self._face_normals[:, np.newaxis, :]
-                self._face_normals_indexed_by_faces = norms
-            return self._face_normals_indexed_by_faces
-        else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+        if self._face_normals is None:
+            vertices = self.get_vertices(indexed='faces')
+            self._face_normals = _compute_face_normals(vertices)
+
+        if indexed == 'faces' and self._face_normals_indexed_by_faces is None:
+            self._face_normals_indexed_by_faces = \
+                _repeat_face_normals_on_corners(self._face_normals)
+
+        return (self._face_normals if indexed is None
+                else self._face_normals_indexed_by_faces)
 
     def get_vertex_normals(self, indexed=None):
         """Get vertex normals
@@ -328,29 +373,20 @@ class MeshData(object):
         normals : ndarray
             The normals.
         """
+        if indexed not in (None, 'faces'):
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
+
         if self._vertex_normals is None:
-            faceNorms = self.get_face_normals()
-            vertFaces = self.get_vertex_faces()
-            self._vertex_normals = np.empty(self._vertices.shape,
-                                            dtype=np.float32)
-            for vindex in xrange(self._vertices.shape[0]):
-                faces = vertFaces[vindex]
-                if len(faces) == 0:
-                    self._vertex_normals[vindex] = (0, 0, 0)
-                    continue
-                norms = faceNorms[faces]  # get all face normals
-                norm = norms.sum(axis=0)  # sum normals
-                renorm = (norm**2).sum()**0.5
-                if renorm > 0:
-                    norm /= renorm
-                self._vertex_normals[vindex] = norm
+            face_normals = self.get_face_normals()
+            faces = self.get_faces()
+            vertices = self.get_vertices()
+            self._vertex_normals = _compute_vertex_normals(face_normals, faces,
+                                                           vertices)
 
         if indexed is None:
             return self._vertex_normals
         elif indexed == 'faces':
             return self._vertex_normals[self.get_faces()]
-        else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
 
     def get_vertex_colors(self, indexed=None):
         """Get vertex colors
@@ -375,7 +411,7 @@ class MeshData(object):
                     self._vertex_colors[self.get_faces()]
             return self._vertex_colors_indexed_by_faces
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
     def get_vertex_values(self, indexed=None):
         """Get vertex colors
@@ -400,7 +436,7 @@ class MeshData(object):
                     self._vertex_values[self.get_faces()]
             return self._vertex_values_indexed_by_faces
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
     def set_vertex_colors(self, colors, indexed=None):
         """Set the vertex color array
@@ -413,7 +449,7 @@ class MeshData(object):
         indexed : str | None
             Should be 'faces' if colors are indexed by faces.
         """
-        colors = _fix_colors(np.asarray(colors))
+        colors = _fix_colors(colors)
         if indexed is None:
             if colors.ndim != 2:
                 raise ValueError('colors must be 2D if indexed is None')
@@ -490,7 +526,7 @@ class MeshData(object):
                     self._face_colors.reshape(Nf, 1, 4)
             return self._face_colors_indexed_by_faces
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
     def set_face_colors(self, colors, indexed=None):
         """Set the face color array
@@ -552,7 +588,7 @@ class MeshData(object):
         self._vertex_faces = []
         self._face_normals = None
         self._vertex_normals = None
-        for i in xrange(faces.shape[0]):
+        for i in range(faces.shape[0]):
             face = faces[i]
             for j in range(face.shape[0]):
                 pt = face[j]
@@ -570,12 +606,10 @@ class MeshData(object):
         self._vertices = np.array(self._vertices, dtype=np.float32)
 
     def get_vertex_faces(self):
-        """
-        List mapping each vertex index to a list of face indices that use it.
-        """
+        """List mapping each vertex index to a list of face indices that use it."""
         if self._vertex_faces is None:
-            self._vertex_faces = [[] for i in xrange(len(self.get_vertices()))]
-            for i in xrange(self._faces.shape[0]):
+            self._vertex_faces = [[] for i in range(len(self.get_vertices()))]
+            for i in range(self._faces.shape[0]):
                 face = self._faces[i]
                 for ind in face:
                     self._vertex_faces[ind].append(i)
@@ -615,7 +649,7 @@ class MeshData(object):
                 raise Exception("MeshData cannot generate edges--no faces in "
                                 "this data.")
         else:
-            raise Exception("Invalid indexing mode. Accepts: None, 'faces'")
+            raise ValueError("Invalid indexing mode. Accepts: None, 'faces'")
 
     def save(self):
         """Serialize this mesh to a string appropriate for disk storage
@@ -658,3 +692,7 @@ class MeshData(object):
             if isinstance(state[k], list):
                 state[k] = np.array(state[k])
             setattr(self, k, state[k])
+
+    def is_empty(self):
+        """Check if any vertices or faces are defined."""
+        return self._faces is None
