@@ -11,12 +11,11 @@ import weakref
 
 from . globject import GLObject
 from ..util import logger
-from ..ext.six import string_types
 
 
 # ------------------------------------------------------------ Buffer class ---
 class Buffer(GLObject):
-    """ Generic GPU buffer.
+    """Generic GPU buffer.
 
     A generic buffer is an interface used to upload data to a GPU array buffer
     (ARRAY_BUFFER or ELEMENT_ARRAY_BUFFER). It keeps track of
@@ -54,16 +53,14 @@ class Buffer(GLObject):
 
     @property
     def nbytes(self):
-        """ Buffer size in bytes """
-
+        """Buffer size in bytes"""
         return self._nbytes
 
     def set_subdata(self, data, offset=0, copy=False):
-        """ Set a sub-region of the buffer (deferred operation).
+        """Set a sub-region of the buffer (deferred operation).
 
         Parameters
         ----------
-
         data : ndarray
             Data to be uploaded
         offset: int
@@ -88,7 +85,7 @@ class Buffer(GLObject):
         self._glir.command('DATA', self._id, offset, data)
 
     def set_data(self, data, copy=False):
-        """ Set data in the buffer (deferred operation).
+        """Set data in the buffer (deferred operation).
 
         This completely resets the size and contents of the buffer.
 
@@ -114,7 +111,7 @@ class Buffer(GLObject):
             self._glir.command('DATA', self._id, 0, data)
 
     def resize_bytes(self, size):
-        """ Resize this buffer (deferred operation).
+        """Resize this buffer (deferred operation).
 
         Parameters
         ----------
@@ -131,7 +128,7 @@ class Buffer(GLObject):
 
 # -------------------------------------------------------- DataBuffer class ---
 class DataBuffer(Buffer):
-    """ GPU data buffer that is aware of data type and elements size
+    """GPU data buffer that is aware of data type and elements size
 
     Parameters
     ----------
@@ -154,28 +151,29 @@ class DataBuffer(Buffer):
         return data
 
     def set_subdata(self, data, offset=0, copy=False, **kwargs):
-        """ Set a sub-region of the buffer (deferred operation).
+        """Set a sub-region of the buffer (deferred operation).
 
         Parameters
         ----------
-
         data : ndarray
             Data to be uploaded
         offset: int
-            Offset in buffer where to start copying data (in bytes)
+            Offset in buffer where to start copying data (i.e. index of
+            starting element).
         copy: bool
             Since the operation is deferred, data may change before
             data is actually uploaded to GPU memory.
             Asking explicitly for a copy will prevent this behavior.
         **kwargs : dict
             Additional keyword arguments.
+
         """
         data = self._prepare_data(data, **kwargs)
         offset = offset * self.itemsize
         Buffer.set_subdata(self, data=data, offset=offset, copy=copy)
 
     def set_data(self, data, copy=False, **kwargs):
-        """ Set data (deferred operation)
+        """Set data (deferred operation)
 
         Parameters
         ----------
@@ -199,36 +197,32 @@ class DataBuffer(Buffer):
 
     @property
     def dtype(self):
-        """ Buffer dtype """
-
+        """Buffer dtype"""
         return self._dtype
 
     @property
     def offset(self):
-        """ Buffer offset (in bytes) relative to base """
-
+        """Buffer offset (in bytes) relative to base"""
         return 0
 
     @property
     def stride(self):
-        """ Stride of data in memory """
+        """Stride of data in memory"""
         return self._stride
 
     @property
     def size(self):
-        """ Number of elements in the buffer """
+        """Number of elements in the buffer"""
         return self._size
 
     @property
     def itemsize(self):
-        """ The total number of bytes required to store the array data """
-
+        """The total number of bytes required to store the array data"""
         return self._itemsize
 
     @property
     def glsl_type(self):
-        """ GLSL declaration strings required for a variable to hold this data.
-        """
+        """GLSL declaration strings required for a variable to hold this data."""
         if self.dtype is None:
             return None
         dtshape = self.dtype[0].shape
@@ -240,7 +234,7 @@ class DataBuffer(Buffer):
         return 'attribute', dtype
 
     def resize_bytes(self, size):
-        """ Resize the buffer (in-place, deferred operation)
+        """Resize the buffer (in-place, deferred operation)
 
         Parameters
         ----------
@@ -255,18 +249,16 @@ class DataBuffer(Buffer):
         self._size = size // self.itemsize
 
     def __getitem__(self, key):
-        """ Create a view on this buffer. """
-
+        """Create a view on this buffer."""
         view = DataBufferView(self, key)
         self._views.add(view)
         return view
 
     def __setitem__(self, key, data):
-        """ Set data (deferred operation) """
-
+        """Set data (deferred operation)"""
         # Setting a whole field of the buffer: only allowed if we have CPU
         # storage. Note this case (key is string) only happen with base buffer
-        if isinstance(key, string_types):
+        if isinstance(key, str):
             raise ValueError("Cannot set non-contiguous data on buffer")
 
         # Setting one or several elements
@@ -299,9 +291,9 @@ class DataBuffer(Buffer):
         elif data.size > stop - start:
             raise ValueError('Data too big to fit GPU data '
                              '(%d > %d-%d).' % (data.size, stop, start))
-        
+
         # Set data
-        offset = start  # * self.itemsize
+        offset = start
         self.set_subdata(data=data, offset=offset, copy=True)
 
     def __repr__(self):
@@ -310,7 +302,7 @@ class DataBuffer(Buffer):
 
 
 class DataBufferView(DataBuffer):
-    """ View on a sub-region of a DataBuffer.
+    """View on a sub-region of a DataBuffer.
 
     Parameters
     ----------
@@ -323,7 +315,6 @@ class DataBufferView(DataBuffer):
 
     Notes
     -----
-
     It is generally not necessary to instantiate this class manually; use
     ``base_buffer[key]`` instead.
     """
@@ -339,7 +330,7 @@ class DataBufferView(DataBuffer):
         self._key = key
         self._stride = base.stride
 
-        if isinstance(key, string_types):
+        if isinstance(key, str):
             self._dtype = base.dtype[key]
             self._offset = base.dtype.fields[key][1]
             self._nbytes = base.size * self._dtype.itemsize
@@ -391,13 +382,12 @@ class DataBufferView(DataBuffer):
 
     @property
     def offset(self):
-        """ Buffer offset (in bytes) relative to base """
-
+        """Buffer offset (in bytes) relative to base"""
         return self._offset
 
     @property
     def base(self):
-        """Buffer base if this buffer is a view on another buffer. """
+        """Buffer base if this buffer is a view on another buffer."""
         return self._base
 
     def resize_bytes(self, size):
@@ -416,7 +406,7 @@ class DataBufferView(DataBuffer):
 
 # ------------------------------------------------------ VertexBuffer class ---
 class VertexBuffer(DataBuffer):
-    """ Buffer for vertex attribute data
+    """Buffer for vertex attribute data
 
     Parameters
     ----------
@@ -424,6 +414,10 @@ class VertexBuffer(DataBuffer):
         Buffer data (optional)
     """
 
+    def __init__(self, data=None, divisor=None):
+        super().__init__(data)
+        self.divisor = divisor
+
     _GLIR_TYPE = 'VertexBuffer'
 
     def _prepare_data(self, data, convert=False):
@@ -459,6 +453,14 @@ class VertexBuffer(DataBuffer):
             self._last_dim = c
         return data
 
+    @property
+    def divisor(self):
+        return self._divisor
+
+    @divisor.setter
+    def divisor(self, value):
+        self._divisor = max(1, int(value)) if value else None
+
 
 def _last_stack_str():
     """Print stack trace from call that didn't originate from here"""
@@ -471,11 +473,10 @@ def _last_stack_str():
 
 # ------------------------------------------------------- IndexBuffer class ---
 class IndexBuffer(DataBuffer):
-    """ Buffer for index data
+    """Buffer for index data
 
     Parameters
     ----------
-
     data : ndarray | None
         Buffer data.
     """
