@@ -9,7 +9,7 @@
 
 The glir module holds the desktop implementation of the GL Intermediate
 Representation (GLIR). Parsing and handling of the GLIR for other platforms
-can be found in external libraries (ex. vispy.js for WebGL).
+can be found in external libraries.
 
 We propose the specification of a simple intermediate representation for
 OpenGL. It provides a means to serialize a visualization, so that the
@@ -22,7 +22,7 @@ another without having to wait for a reply. Only in the event of an
 error information needs to go in the other direction, but this can be
 done asynchronously.
 
-The purpose for Vispy is to allow the usage gloo (our high level object
+The purpose for GLIR has been to allow the usage gloo (our high level object
 oriented interface to OpenGL), while executing the visualization in the
 browser (via JS/WebGL). The fact that the stream of commands is
 one-directional is essential to realize reactive visualizations.
@@ -173,17 +173,20 @@ DRAW
 
 ::
 
-   ('DRAW', <program_id>, <mode:str>, <selection:tuple>)
-   # Example: Draw 100 lines
-   ('DRAW', 4, 'lines', (0, 100))
+   ('DRAW', <program_id>, <mode:str>, <selection:tuple>, <instances:int>)
+   # Example: Draw 100 lines with non-instanced rendering
+   ('DRAW', 4, 'lines', (0, 100), 1)
    # Example: Draw 100 lines using index buffer with id 5
-   ('DRAW', 4, 'points', (5, 'unsigned_int', 100))
+   ('DRAW', 4, 'points', (5, 'unsigned_int', 100), 1)
+   # Example: Draw a mesh with 10 vertices 20 times using instanced rendering
+   ('DRAW', 2, 'mesh', (0, 10), 20)
 
 Applies to: Program
 
 This command is used to draw the program. It has a ``mode`` argument
-which can be 'points', 'lines', 'line_strip', 'line_loop', 'triangles',
-'triangle_strip', or 'triangle_fan' (case insensitive).
+which can be 'points', 'lines', 'line_strip', 'line_loop', 'lines_adjacency',
+'line_strip_adjacency', 'triangles', 'triangle_strip', or 'triangle_fan'
+(case insensitive).
 
 If the ``selection`` argument has two elements, it contains two integers
 ``(start, count)``. If it has three elements, it contains
@@ -361,16 +364,18 @@ import sys
 import re
 import json
 import weakref
-from distutils.version import LooseVersion
+from packaging.version import Version
 
 import numpy as np
 
 from . import gl
-from ..ext.six import string_types
 from ..util import logger
 
 # TODO: expose these via an extension space in .gl?
 _internalformats = [
+    gl.Enum('GL_DEPTH_COMPONENT', 6402),
+    gl.Enum('GL_DEPTH_COMPONENT16', 33189),
+    gl.Enum('GL_DEPTH_COMPONENT32_OES', 33191),
     gl.Enum('GL_RED', 6403),
     gl.Enum('GL_R', 8194),
     gl.Enum('GL_R8', 33321),
@@ -391,7 +396,16 @@ _internalformats = [
     gl.Enum('GL_RGBA8', 32856),
     gl.Enum('GL_RGBA16', 32859),
     gl.Enum('GL_RGBA16F', 34842),
-    gl.Enum('GL_RGBA32F', 34836)
+    gl.Enum('GL_RGBA32F', 34836),
+    # extended formats (not currently supported)
+    # gl.Enum('GL_R32I', 33333),
+    # gl.Enum('GL_RG32I', 33339),
+    # gl.Enum('GL_RGB32I', 36227),
+    # gl.Enum('GL_RGBA32I', 36226),
+    # gl.Enum('GL_R32UI', 33334),
+    # gl.Enum('GL_RG32UI', 33340),
+    # gl.Enum('GL_RGB32UI', 36209),
+    # gl.Enum('GL_RGBA32UI', 36208),
 ]
 _internalformats = dict([(enum.name, enum) for enum in _internalformats])
 
@@ -404,9 +418,8 @@ JUST_DELETED = 'JUST_DELETED'
 
 
 def as_enum(enum):
-    """ Turn a possibly string enum into an integer enum.
-    """
-    if isinstance(enum, string_types):
+    """Turn a possibly string enum into an integer enum."""
+    if isinstance(enum, str):
         try:
             enum = getattr(gl, 'GL_' + enum.upper())
         except AttributeError:
@@ -436,6 +449,7 @@ class _GlirQueueShare(object):
     of GL objects, when some queues may only be joined at the last possible
     moment.
     """
+
     def __init__(self, queue):
         self._commands = []  # local commands
         self._verbose = False
@@ -443,20 +457,19 @@ class _GlirQueueShare(object):
         self._associations = weakref.WeakKeyDictionary({queue: None})
 
     def command(self, *args):
-        """ Send a command. See the command spec at:
+        """Send a command. See the command spec at:
         https://github.com/vispy/vispy/wiki/Spec.-Gloo-IR
         """
         self._commands.append(args)
 
     def set_verbose(self, verbose):
-        """ Set verbose or not. If True, the GLIR commands are printed
-        right before they get parsed. If a string is given, use it as
-        a filter.
+        """Set verbose or not. If True, the GLIR commands are printed right before they get parsed.
+        If a string is given, use it as a filter.
         """
         self._verbose = verbose
 
     def show(self, filter=None):
-        """ Print the list of commands currently in the queue. If filter is
+        """Print the list of commands currently in the queue. If filter is
         given, print only commands that match the filter.
         """
         for command in self._commands:
@@ -478,7 +491,7 @@ class _GlirQueueShare(object):
             print(tuple(t))
 
     def clear(self):
-        """ Pop the whole queue (and associated queues) and return a
+        """Pop the whole queue (and associated queues) and return a
         list of commands.
         """
         commands = self._commands
@@ -486,15 +499,14 @@ class _GlirQueueShare(object):
         return commands
 
     def flush(self, parser):
-        """ Flush all current commands to the GLIR interpreter.
-        """
+        """Flush all current commands to the GLIR interpreter."""
         if self._verbose:
             show = self._verbose if isinstance(self._verbose, str) else None
             self.show(show)
         parser.parse(self._filter(self.clear(), parser))
 
     def _filter(self, commands, parser):
-        """ Filter DATA/SIZE commands that are overridden by a
+        """Filter DATA/SIZE commands that are overridden by a
         SIZE command.
         """
         resized = set()
@@ -510,7 +522,7 @@ class _GlirQueueShare(object):
 
 
 class GlirQueue(object):
-    """ Representation of a queue of GLIR commands
+    """Representation of a queue of GLIR commands
 
     One instance of this class is attached to each context object, and
     to each gloo object. Internally, commands are stored in a shared queue
@@ -522,26 +534,27 @@ class GlirQueue(object):
     context.shared. The parser can interpret the commands in Python,
     send them to a browser, etc.
     """
+
     def __init__(self):
         # We do not actually queue any commands here, but on a shared queue
         # object that may be joined with others as queues are associated.
         self._shared = _GlirQueueShare(self)
 
     def command(self, *args):
-        """ Send a command. See the command spec at:
+        """Send a command. See the command spec at:
         https://github.com/vispy/vispy/wiki/Spec.-GLIR
         """
         self._shared.command(*args)
 
     def set_verbose(self, verbose):
-        """ Set verbose or not. If True, the GLIR commands are printed
+        """Set verbose or not. If True, the GLIR commands are printed
         right before they get parsed. If a string is given, use it as
         a filter.
         """
         self._shared.set_verbose(verbose)
 
     def clear(self):
-        """ Pop the whole queue (and associated queues) and return a
+        """Pop the whole queue (and associated queues) and return a
         list of commands.
         """
         return self._shared.clear()
@@ -567,8 +580,7 @@ class GlirQueue(object):
         queue._shared = self._shared
 
     def flush(self, parser):
-        """ Flush all current commands to the GLIR interpreter.
-        """
+        """Flush all current commands to the GLIR interpreter."""
         self._shared.flush(parser)
 
 
@@ -647,20 +659,17 @@ def convert_shader(backend_type, shader):
 
 
 def as_es2_command(command):
-    """ Modify a desktop command so it works on es2.
-    """
-
+    """Modify a desktop command so it works on es2."""
     if command[0] == 'FUNC':
         return (command[0], re.sub(r'^gl([A-Z])',
-                lambda m: m.group(1).lower(), command[1])) + command[2:]
+                                   lambda m: m.group(1).lower(), command[1])) + command[2:]
     elif command[0] == 'UNIFORM':
         return command[:-1] + (command[-1].tolist(),)
     return command
 
 
 class BaseGlirParser(object):
-    """ Base clas for GLIR parsers that can be attached to a GLIR queue.
-    """
+    """Base class for GLIR parsers that can be attached to a GLIR queue."""
 
     def __init__(self):
         self.capabilities = dict(
@@ -669,26 +678,25 @@ class BaseGlirParser(object):
         )
 
     def is_remote(self):
-        """ Whether the code is executed remotely. i.e. gloo.gl cannot
+        """Whether the code is executed remotely. i.e. gloo.gl cannot
         be used.
         """
         raise NotImplementedError()
 
     @property
     def shader_compatibility(self):
-        """ Whether to convert shading code. Valid values are 'es2' and
+        """Whether to convert shading code. Valid values are 'es2' and
         'desktop'. If None, the shaders are not modified.
         """
         raise NotImplementedError()
 
     def parse(self, commands):
-        """ Parse the GLIR commands. Or sent them away.
-        """
+        """Parse the GLIR commands. Or sent them away."""
         raise NotImplementedError()
 
 
 class GlirParser(BaseGlirParser):
-    """ A class for interpreting GLIR commands using gloo.gl
+    """A class for interpreting GLIR commands using gloo.gl
 
     We make use of relatively light GLIR objects that are instantiated
     on CREATE commands. These objects are stored by their id in a
@@ -723,7 +731,7 @@ class GlirParser(BaseGlirParser):
 
     @property
     def shader_compatibility(self):
-        """Type of shader compatibility """
+        """Type of shader compatibility"""
         if '.es' in gl.current_backend.__name__:
             return 'es2'
         else:
@@ -733,8 +741,7 @@ class GlirParser(BaseGlirParser):
         return False
 
     def _parse(self, command):
-        """ Parse a single command.
-        """
+        """Parse a single command."""
         cmd, id_, args = command[0], command[1], command[2:]
 
         if cmd == 'CURRENT':
@@ -803,9 +810,7 @@ class GlirParser(BaseGlirParser):
                 logger.warning('Invalid GLIR command %r' % cmd)
 
     def parse(self, commands):
-        """ Parse a list of commands.
-        """
-
+        """Parse a list of commands."""
         # Get rid of dummy objects that represented deleted objects in
         # the last parsing round.
         to_delete = []
@@ -819,14 +824,11 @@ class GlirParser(BaseGlirParser):
             self._parse(command)
 
     def get_object(self, id_):
-        """ Get the object with the given id or None if it does not exist.
-        """
+        """Get the object with the given id or None if it does not exist."""
         return self._objects.get(id_, None)
 
     def _gl_initialize(self):
-        """ Deal with compatibility; desktop does not have sprites
-        enabled by default. ES has.
-        """
+        """Deal with compatibility; desktop does not have sprites enabled by default. ES has."""
         if '.es' in gl.current_backend.__name__:
             pass  # ES2: no action required
         else:
@@ -847,8 +849,10 @@ class GlirParser(BaseGlirParser):
             else:
                 this_version = this_version[0]
 
-            this_version = LooseVersion(this_version)
-            if this_version < '2.1':
+            if not this_version:
+                logger.warning("OpenGL version could not be determined, which "
+                               "might be a sign that OpenGL is not loaded correctly.")
+            elif Version(this_version) < Version('2.1'):
                 if os.getenv('VISPY_IGNORE_OLD_VERSION', '').lower() != 'true':
                     logger.warning('OpenGL version 2.1 or higher recommended, '
                                    'got %s. Some functionality may fail.'
@@ -862,7 +866,7 @@ def glir_logger(parser_cls, file_or_filename):
         def __init__(self, *args, **kwargs):
             parser_cls.__init__(self, *args, **kwargs)
 
-            if isinstance(file_or_filename, string_types):
+            if isinstance(file_or_filename, str):
                 self._file = open(file_or_filename, 'w')
             else:
                 self._file = file_or_filename
@@ -885,7 +889,7 @@ def glir_logger(parser_cls, file_or_filename):
     return cls
 
 
-## GLIR objects
+# GLIR objects
 
 class GlirObject(object):
     def __init__(self, parser, id_):
@@ -961,7 +965,7 @@ class GlirShader(GlirObject):
         return '\n'.join(results)
 
     def _parse_error(self, error):
-        """ Parses a single GLSL error and extracts the linenr and description
+        """Parses a single GLSL error and extracts the linenr and description
         Other GLIR implementations may omit this.
         """
         error = str(error)
@@ -999,7 +1003,8 @@ class GlirGeometryShader(GlirShader):
 
     def __init__(self, *args, **kwargs):
         if not hasattr(gl, 'GL_GEOMETRY_SHADER'):
-            raise RuntimeError("GL2 backend does not support geometry shaders."
+            raise RuntimeError(gl.current_backend.__name__ +
+                               " backend does not support geometry shaders."
                                " Try gloo.gl.use_gl('gl+').")
         GlirGeometryShader._target = gl.GL_GEOMETRY_SHADER
         GlirShader.__init__(self, *args, **kwargs)
@@ -1061,7 +1066,7 @@ class GlirProgram(GlirObject):
         gl.glDeleteProgram(self._handle)
 
     def activate(self):
-        """ Avoid overhead in calling glUseProgram with same arg.
+        """Avoid overhead in calling glUseProgram with same arg.
         Warning: this will break if glUseProgram is used somewhere else.
         Per context we keep track of one current program.
         """
@@ -1070,7 +1075,7 @@ class GlirProgram(GlirObject):
             gl.glUseProgram(self._handle)
 
     def deactivate(self):
-        """ Avoid overhead in calling glUseProgram with same arg.
+        """Avoid overhead in calling glUseProgram with same arg.
         Warning: this will break if glUseProgram is used somewhere else.
         Per context we keep track of one current program.
         """
@@ -1079,7 +1084,7 @@ class GlirProgram(GlirObject):
             gl.glUseProgram(0)
 
     def set_shaders(self, vert, frag):
-        """ This function takes care of setting the shading code and
+        """This function takes care of setting the shading code and
         compiling+linking it into a working program object that is ready
         to use.
         """
@@ -1093,14 +1098,13 @@ class GlirProgram(GlirObject):
         self.link_program()
 
     def attach(self, id_):
-        """ Attach a shader to this program.
-        """
+        """Attach a shader to this program."""
         shader = self._parser.get_object(id_)
         gl.glAttachShader(self._handle, shader.handle)
         self._attached_shaders.append(shader)
 
     def link_program(self):
-        """ Link the complete program and check.
+        """Link the complete program and check.
 
         All shaders are detached and deleted if the program was successfully
         linked.
@@ -1123,7 +1127,7 @@ class GlirProgram(GlirObject):
         self._linked = True
 
     def _get_active_attributes_and_uniforms(self):
-        """ Retrieve active attributes and uniforms to be able to check that
+        """Retrieve active attributes and uniforms to be able to check that
         all uniforms/attributes are set by the user.
         Other GLIR implementations may omit this.
         """
@@ -1146,12 +1150,11 @@ class GlirProgram(GlirObject):
                         container.append(('%s[%d]' % (name, i), gtype))
                 else:
                     container.append((name, gtype))
-        #return attributes, uniforms
+        # return attributes, uniforms
         return set([v[0] for v in attributes] + [v[0] for v in uniforms])
 
     def set_texture(self, name, value):
-        """ Set a texture sampler. Value is the id of the texture to link.
-        """
+        """Set a texture sampler. Value is the id of the texture to link."""
         if not self._linked:
             raise RuntimeError('Cannot set uniform when program has no code')
         # Get handle for the uniform, first try cache
@@ -1183,8 +1186,7 @@ class GlirProgram(GlirObject):
             gl.glUniform1i(handle, unit)
 
     def set_uniform(self, name, type_, value):
-        """ Set a uniform value. Value is assumed to have been checked.
-        """
+        """Set a uniform value. Value is assumed to have been checked."""
         if not self._linked:
             raise RuntimeError('Cannot set uniform when program has no code')
         # Get handle for the uniform, first try cache
@@ -1223,9 +1225,8 @@ class GlirProgram(GlirObject):
             # Regular uniform
             func(handle, count, value)
 
-    def set_attribute(self, name, type_, value):
-        """ Set an attribute value. Value is assumed to have been checked.
-        """
+    def set_attribute(self, name, type_, value, divisor=None):
+        """Set an attribute value. Value is assumed to have been checked."""
         if not self._linked:
             raise RuntimeError('Cannot set attribute when program has no code')
         # Get handle for the attribute, first try cache
@@ -1251,7 +1252,7 @@ class GlirProgram(GlirObject):
             funcname = self.ATYPEMAP[type_]
             func = getattr(gl, funcname)
             # Set data
-            self._attributes[name] = 0, handle, func, value[1:]
+            self._attributes[name] = 0, handle, func, value[1:], divisor
         else:
             # Get meta data
             vbo_id, stride, offset = value
@@ -1265,7 +1266,7 @@ class GlirProgram(GlirObject):
             # Set data
             func = gl.glVertexAttribPointer
             args = size, gtype, gl.GL_FALSE, stride, offset
-            self._attributes[name] = vbo.handle, handle, func, args
+            self._attributes[name] = vbo.handle, handle, func, args, divisor
 
     def _pre_draw(self):
         self.activate()
@@ -1274,11 +1275,13 @@ class GlirProgram(GlirObject):
             gl.glActiveTexture(gl.GL_TEXTURE0 + unit)
             gl.glBindTexture(tex_target, tex_handle)
         # Activate attributes
-        for vbo_handle, attr_handle, func, args in self._attributes.values():
+        for vbo_handle, attr_handle, func, args, divisor in self._attributes.values():
             if vbo_handle:
                 gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vbo_handle)
                 gl.glEnableVertexAttribArray(attr_handle)
                 func(attr_handle, *args)
+                if divisor is not None:
+                    gl.glVertexAttribDivisor(attr_handle, divisor)
             else:
                 gl.glBindBuffer(gl.GL_ARRAY_BUFFER, 0)
                 gl.glDisableVertexAttribArray(attr_handle)
@@ -1291,8 +1294,8 @@ class GlirProgram(GlirObject):
     def _validate(self):
         # Validate ourselves
         if self._unset_variables:
-            logger.info('Program has unset variables: %r' %
-                        self._unset_variables)
+            logger.warning('Program has unset variables: %r' %
+                           self._unset_variables)
         # Validate via OpenGL
         gl.glValidateProgram(self._handle)
         if not gl.glGetProgramParameter(self._handle,
@@ -1308,19 +1311,28 @@ class GlirProgram(GlirObject):
             gl.glBindTexture(GL_TEXTURE_3D, 0)
             gl.glBindTexture(GL_TEXTURE_1D, 0)
 
-        #Deactivate program - should not be necessary. In single-program
-        #apps it would not even make sense.
-        #self.deactivate()
+        # Deactivate program - should not be necessary. In single-program
+        # apps it would not even make sense.
+        # self.deactivate()
 
-    def draw(self, mode, selection):
-        """ Draw program in given mode, with given selection (IndexBuffer or
+    def draw(self, mode, selection, instances=1):
+        """Draw program in given mode, with given selection (IndexBuffer or
         first, count).
         """
         if not self._linked:
             raise RuntimeError('Cannot draw program if code has not been set')
         # Init
         gl.check_error('Check before draw')
-        mode = as_enum(mode)
+        try:
+            mode = as_enum(mode)
+        except ValueError:
+            if mode == 'lines_adjacency' or mode == 'line_strip_adjacency':
+                raise RuntimeError(gl.current_backend.__name__ +
+                                   " backend does not support lines_adjacency"
+                                   " and line_strip_adjacency primitives."
+                                   " Try gloo.gl.use_gl('gl+').")
+            raise
+
         # Draw
         if len(selection) == 3:
             # Selection based on indices
@@ -1329,14 +1341,20 @@ class GlirProgram(GlirObject):
                 self._pre_draw()
                 ibuf = self._parser.get_object(id_)
                 ibuf.activate()
-                gl.glDrawElements(mode, count, as_enum(gtype), None)
+                if instances > 1:
+                    gl.glDrawElementsInstanced(mode, count, as_enum(gtype), None, instances)
+                else:
+                    gl.glDrawElements(mode, count, as_enum(gtype), None)
                 ibuf.deactivate()
         else:
             # Selection based on start and count
             first, count = selection
             if count:
                 self._pre_draw()
-                gl.glDrawArrays(mode, first, count)
+                if instances > 1:
+                    gl.glDrawArraysInstanced(mode, first, count, instances)
+                else:
+                    gl.glDrawArrays(mode, first, count)
         # Wrap up
         gl.check_error('Check after draw')
         self._post_draw()
@@ -1372,7 +1390,7 @@ class GlirBuffer(GlirObject):
 
         # Determine whether to check errors to try handling the ATI bug
         check_ati_bug = ((not self._bufferSubDataOk) and
-                         (gl.current_backend is gl.gl2) and
+                         (gl.current_backend.__name__.split(".")[-1] == "gl2") and
                          sys.platform.startswith('win'))
 
         # flush any pending errors
@@ -1700,7 +1718,7 @@ class GlirRenderBuffer(GlirObject):
         gl.glBindRenderbuffer(gl.GL_RENDERBUFFER, 0)
 
     def set_size(self, shape, format):
-        if isinstance(format, string_types):
+        if isinstance(format, str):
             format = GlirFrameBuffer._formats[format][1]
         if (shape, format) != self._shape_format:
             self._shape_format = shape, format
@@ -1717,7 +1735,7 @@ class GlirFrameBuffer(GlirObject):
                 'stencil': (gl.GL_STENCIL_ATTACHMENT, gl.GL_STENCIL_INDEX8)}
 
     def create(self):
-        #self._parser._fb_stack = [0]  # To keep track of active FB
+        # self._parser._fb_stack = [0]  # To keep track of active FB
         self._handle = gl.glCreateFramebuffer()
         self._validated = False
 
