@@ -36,10 +36,12 @@ class Fig(SceneCanvas):
     PlotWidget : the axis widget for plotting
     SceneCanvas : the super class
     """
-    def __init__(self, bgcolor='w', size=(800, 600), show=True, **kwargs):
+
+    def __init__(self, bgcolor='w', size=(800, 600), show=True,
+                 keys='interactive', **kwargs):
         self._plot_widgets = []
         self._grid = None  # initialize before the freeze occurs
-        super(Fig, self).__init__(bgcolor=bgcolor, keys='interactive',
+        super(Fig, self).__init__(bgcolor=bgcolor, keys=keys,
                                   show=show, size=size, **kwargs)
         self._grid = self.central_widget.add_grid()
         self._grid._default_class = PlotWidget
